# =========================================================================== #
# Gemspec for Project EnvironmentInformation.
# =========================================================================== #
require 'environment_information/version/version.rb'

Gem::Specification.new { |s|

  s.name    = 'environment_information'
  s.version = EnvironmentInformation::VERSION
  s.date    = Time.now.strftime('%Y-%m-%d')

  DESCRIPTION = <<-EOF

The gem environment_information is able to provide information about the
computer host system, in particular on Linux systems, if ruby is available
on that computer. Commonly used programs will be queried for their current
version number (and whether they exist) on the target computer. If these
programs do happen to exist then the corresponding version will be
displayed, on the commandline.

This is somewhat similar to the "Linux from Scratch" project, which uses
a shell script to display the version of many commonly used programs.
As I prefer ruby over shell scripts, I wrote this somewhat equivalent
project.

For more information and documentation about this project, consider
looking at the homepage of this gem, at

  https://rubygems.org/gems/environment_information

and look at the documentation on the bottom right side,
called "Documentation".

  EOF
  
  s.summary     = DESCRIPTION
  s.description = DESCRIPTION
  
  # =============================================================== #
  # Show this when a user installs this project.
  # =============================================================== #
  s.post_install_message = <<-EOF

You can run this little script by issuing:

    envi

on the commandline. (The script envi is part of this gem and can
be found at the subdirectory bin/envi of the gem.)

Within ruby code, you can require the project and then display
information too, via:

    require 'environment_information'
    EnvironmentInformation.run

For the (documented) options, you can do either of the following:

    envi --help
    envi HELP

This will feedback the options that are available.

To get the full output of just about every program that is checked
by this gem, try any of the following invocation:

    envi --all
    envi ALL

(Lowercase works fine just as well. The commandline options that
include two '-' characters are, internally, the preferred variants.)

To obtain an individual entry, do something such as this:

    envi gtk
    envi pango
    envi gcr

To disable colours, do either one of these:

    envi --nocolours
    envi --no-colours
    envi --no-colors
    envi --nocolors
    envi nocolours
    envi NOCOLOURS

(Both UK and US spelling is supported, so colour works just as
well as color.)

If all of this may become too tedious to type, I just use an
alias called ALL, which stands short for "envi all" or 
"envi show_all".

Another option, both of which are synonymous to each other,
is to compare all existing programs on a given host system
with the version registered in the RBT project:

    envi ALL REALLY_ALL
    envi --compare_programs
    envi compare_programs

If you wish to show only the OS information, do:

    envi --os?

For more documentation about the project, have a look at:

    #{EnvironmentInformation::URL_TO_THE_DOCUMENTATION}

  EOF

  s.authors    = ['Robert A. Heiler']
  s.email      = 'shevegen@gmail.com'
  s.files      = Dir['**/*']
  s.licenses   = 'GPL-2.0'
  s.homepage   = 'http://rubygems.org/gems/environment_information'
  s.executables = Dir['bin/*'].map { |f| File.basename(f) }

  s.required_ruby_version     = '>= '+RUBY_VERSION
  s.required_rubygems_version = '>= '+Gem::VERSION
  s.rubygems_version          = '>= '+Gem::VERSION

  # ========================================================================= #
  # Dependencies of this project:
  # ========================================================================= #
  s.add_dependency 'colours'
  s.add_dependency 'opn'

}